#! /bin/sh
#
#  Shell script to adjust the version numbers and dates in cbar.h,
#  dllver.rc, readme._tx, cbar._tx and makefile.ver.


if [ $# -lt 3 -o $# -gt 4 ]; then
   echo "Usage: fixver major_num sub_num wip_num [comment]" 1>&2
   echo "Example: fixver 3 9 1 WIP" 1>&2
   exit 1
fi

# get the version and date strings in a nice format
if [ $# -eq 3 ]; then
   verstr="$1.$2.$3"
else
   verstr="$1.$2.$3 ($4)"
fi

year=$(date +%Y)
month=$(date +%m)
day=$(date +%d)
datestr="$(date +%b) $day, $year"

# patch include/cbar.h
echo "s/\#define CBAR_VERSION_MAJOR .*/\#define CBAR_VERSION_MAJOR      $1/" > fixver.sed
echo "s/\#define CBAR_VERSION_MINOR .*/\#define CBAR_VERSION_MINOR      $2/" >> fixver.sed
echo "s/\#define CBAR_VERSION_PATCH .*/\#define CBAR_VERSION_PATCH      $3/" >> fixver.sed
echo "s/\#define CBAR_VERSION_STRING .*/\#define CBAR_VERSION_STRING     \"$verstr\"/" >> fixver.sed
echo "s/\#define CBAR_DATE .*/\#define CBAR_DATE               $year$month$day    \/\* yyyymmdd \*\//" >> fixver.sed
echo "s/\#define CBAR_DATE_STRING .*/\#define CBAR_DATE_STRING \"$datestr\"/" >> fixver.sed
echo "Patching include/cbar.h..."
cp include/cbar.h fixver.tmp
sed -f fixver.sed fixver.tmp > include/cbar.h

echo "s/@manh=\"version [^\"]*\"/@manh=\"version $verstr\"/" >> fixver.sed
echo "s/The Manual of CBAR.*/The Manual of CBAR $verstr/" >> fixver.sed
echo "Patching docs/cbar._tx..."
cp docs/cbar._tx fixver.tmp
sed -f fixver.sed fixver.tmp > docs/cbar._tx

# patch misc/makefile.ver
echo "s/LIB_VERSION = .*/LIB_VERSION = $1$2/" > fixver.sed
echo "s/shared_version = .*/shared_version = $1.$2.$3/" >> fixver.sed
echo "s/shared_major_minor = .*/shared_major_minor = $1.$2/" >> fixver.sed
echo "Patching misc/makefile.ver..."
cp misc/makefile.ver fixver.tmp
sed -f fixver.sed fixver.tmp > misc/makefile.ver

# patch misc/zipup.sh
echo "s/zipversion=.*/zipversion=$1$2$3/" > fixver.sed
echo "s/tgzversion=.*/tgzversion=$1.$2.$3/" >> fixver.sed
echo "Patching misc/zipup.sh..."
cp misc/zipup.sh fixver.tmp
sed -f fixver.sed fixver.tmp > misc/zipup.sh

# clean up after ourselves
rm fixver.sed fixver.tmp

echo "Done!"
