# zipup.sh:
# Script to generate the distributions.
#
# By Christer Sandberg (ported from Adime by Sven Sandberg)
#
# Requires zip, tar, gzip, rm, cp, and sed.
#
# whisped (my own tool for converting tabs, newline characters and the like)
# will be used if it is available, but it's no big deal if this is not the
# case.
##

# note: these are patched by fixver.sh script.
zipversion=104
tgzversion=1.0.4

if [ "$1" != "-q" ]; then

echo clean up...
# Make unix line endings
./fix.sh unix --dtou

# Dont generate dependencies if a parameter was given.
# This is because I often realize that I missed something essential _after_
# calling zipup.sh, so it saves lots of time if the dependencies don't need
# to be rebuilt
if [ "$1" = "--no-depend" ]; then
   echo skip generating dependencies
else
   echo generate dll stuff...
   ./misc/fixdll.sh
   echo generate dependencies...
   #But don't alter the line endings (--quick ensures that)
   ./fix.sh mingw32 --quick
   make depend

   ./fix.sh msvc --quick
   make depend

   ./fix.sh unix --quick
   make depend

   ./fix.sh djgpp --quick
   make depend
fi

echo generate other files for the distribution...
make docs "LFN=y"
make dat

fi

echo copy default makefile...
cp misc/emptymak makefile

files="lib/*/tmpfile.txt \
       makefile misc/makefile* misc/emptymak \
       fix* misc/*.sh \
       misc/dllsyms.lst misc/scanexp.c \
       obj/*/*/*.dep lib/*/*.def \
       readme.txt readme.html docs/readme.info \
       docs/*._tx \
       src/*.c src/*.h src/*.t src/*.ht \
       examples/*.c examples/*.cfg \
       resource/*.dat resource/cbarlabl  \
       include/*.h include/cbar/*.h"

afiles=`echo $files       | sed -e"s/\([^ ][^ ]*\)/ cbar\/\1/g"`


#############################################################################
echo remove old archives...
rm -f cbar$zipversion.zip
rm -f cbar-$tgzversion.tar.gz

# Line endings are still unix in all files, including bat-files. To make the tar archive useful also for
# Windows users, we change the line endings to DOS/Windows already before packing the tar. This should be
# no problem for unix users since the bat file has no meaning in unix.
cp "fix.bat" _tmpfile
perl -p -i -e "s/([^\r]|^)\n/\1\r\n/" _tmpfile
touch -r "fix.bat" _tmpfile
mv _tmpfile "fix.bat"

echo create tar.gz archives...
cd ..
tar -cv $afiles | gzip -9  > cbar/cbar-$tgzversion.tar.gz
cd cbar

echo create zip archives...
# Change line endings from unix to DOS/Windows before packing the zip archives
./fix.sh djgpp --utod
cd ..
zip -9 cbar/cbar$zipversion.zip $afiles
cd cbar

echo "Archives finished!"

# Restore correct unix line endings
./fix.sh unix --dtou

# fix.sh omits to change line endings in the bat-file, so we need to change it separately because we want the local
# copy to have the unix style to have the right endings when committing to cvs next time.
mv "fix.bat" _tmpfile
tr -d '\015' < _tmpfile > "fix.bat"
touch -r _tmpfile "fix.bat"
rm _tmpfile

echo all done!
