/* Module BCS.C
   This file contains the dialogue functions for adpting to barcodes. */

#include <allegro.h>

#include <cgui.h>
#include <cgui/mem.h>

#include "cbar.h"
#include "bcs.h"
#include "bcs.ht"
#include "setting.h"
#include "unit.h"
#include "labldata.h"

typedef struct t_barcodes {
   t_rsetting setting;
   int nrspeakers;              /* Counter to keep track of the number of
                                   readers that outputs to external speakers */
   int n;                       /* Number of readers */
   struct t_unit **units;             /* Array of reader units */
} t_barcodes;

t_barcodes barcodes;

char *_cbc_bar_section_name = "Bar-codes";
static char *bar_dbid_n = "Nr-readers";
static char *bar_dbid_logging = "Logging";
static char *bar_dbid_logwin_open = "Logg_window_open";
static char *bar_dbid_read_check = "Read-check";
char *_cbc_bar_dbid_write_check = "Write-check";
char *_cbc_bar_dbid_nr_digits = "Digits";

static void StoreSettings(void)
{
   int i;

   set_config_int(_cbc_bar_section_name, bar_dbid_n, barcodes.n);
   set_config_int(_cbc_bar_section_name, bar_dbid_logging, barcodes.setting.logging);
   set_config_int(_cbc_bar_section_name, bar_dbid_logwin_open, barcodes.setting.logwin_open);
   set_config_int(_cbc_bar_section_name, _cbc_bar_dbid_nr_digits, barcodes.setting.len);
   set_config_int(_cbc_bar_section_name, bar_dbid_read_check, barcodes.setting.readchk);
   set_config_int(_cbc_bar_section_name, _cbc_bar_dbid_write_check, barcodes.setting.writechk);
   for (i = 0; i < barcodes.n; i++) {
      _CbarStoreUnitConfig(barcodes.units[i], i, _cbc_bar_section_name);
   }
}

static void InstallCleanup(void)
{
   static int done = 0;

   if (!done) {
      done = 1;
      atexit(CbarDeInit);
   }
}
/* The data
   -------- */

static void LoadBarCodesConfig(t_barcodes *b)
{
   int i, n;

   n = get_config_int(_cbc_bar_section_name, bar_dbid_n, 0);
   for (i = 0; i < n; i++)
      CbarAddUnit();
   if (b->n) {
      b->setting.len = get_config_int(_cbc_bar_section_name, _cbc_bar_dbid_nr_digits, 0);
      b->setting.logging = get_config_int(_cbc_bar_section_name, bar_dbid_logging, 0);
      b->setting.logwin_open = get_config_int(_cbc_bar_section_name, bar_dbid_logwin_open, 0);
      b->setting.readchk = get_config_int(_cbc_bar_section_name, bar_dbid_read_check, 1);
      b->setting.writechk = get_config_int(_cbc_bar_section_name, _cbc_bar_dbid_write_check, 1);
      for (i = 0; i < b->n; i++)
         _CbarLoadUnitConfig(b->units[i], i, _cbc_bar_section_name);
   }
}

extern void _CbarInsertUnit(struct t_unit *bcu)
{
   barcodes.units = ResizeMem(struct t_unit *, barcodes.units, ++barcodes.n);
   barcodes.units[barcodes.n - 1] = bcu;
}

extern struct t_unit *_CbarGetUnit(int index)
{
   if (index < barcodes.n)
      return barcodes.units[index];
   return NULL;
}

extern void _CbarRegisterUseSound(int want_left, int want_right)
{
   if (want_left || want_right) {
      if (barcodes.nrspeakers == 0)
         _CbarSetExternalSpeakers(1);
      if (want_left)
         barcodes.nrspeakers++;
      if (want_right)
         barcodes.nrspeakers++;
   }
}

extern void _CbarUnRegisterUseSound(int used_left, int used_right)
{
   if (used_left || used_right) {
      if (used_left)
         barcodes.nrspeakers--;
      if (used_right)
         barcodes.nrspeakers--;
      if (barcodes.nrspeakers <= 0) {
         barcodes.nrspeakers = 0;
         _CbarSetExternalSpeakers(0);
      }
   }
}

extern void _CbarRemoveUnit(struct t_unit *bcu)
{
   int i;
   for (i = 0; i < barcodes.n; i++)
      if (barcodes.units[i-1] == bcu)
         break;
   for (i++; i < barcodes.n; i++)
      barcodes.units[i-1] = barcodes.units[i];
   barcodes.n--;
}

/* Application interface (data access): */

extern int CbarActivate(int seqnr, void (*CallBack) (int, char *, void *),
                            void *data, int mode)
{
   int i;
   struct t_unit *bcu;

   if (seqnr < 0) {
      for (i = 0; i < barcodes.n; i++)
         _CbarActivateUnit(barcodes.units[i], CallBack, data, mode, i);
   } else {
      bcu = _CbarGetUnit(seqnr);
      if (bcu == NULL)
         return 0;
      _CbarActivateUnit(bcu, CallBack, data, mode, seqnr);
   }
   return 1;
}

extern int CbarLoadInit(int mode)
{
   InstallCleanup();
   CbarDeInit();
   LoadBarCodesConfig(&barcodes);
   if ((mode&CBAR_NO_SAVESTATE_LOG)==0 && barcodes.setting.logging)
      CbarLogErrors(CBAR_LOG_ON);
   if ((mode&CBAR_NO_SAVESTATE_LOG_WIN)==0 && barcodes.setting.logwin_open)
      CbarShowErrorLog();
   return barcodes.n;
}

extern int CbarAddUnit(void)
{
   struct t_unit *bcu;

   InstallCleanup();
   bcu = _CbarCreateUnit(&barcodes.setting);
   _CbarInsertUnit(bcu);
   return barcodes.n - 1;
}

extern int CbarSetup(int codelen, int readchk, int writechk)
{
   barcodes.setting.len = codelen;
   barcodes.setting.readchk = readchk;
   barcodes.setting.writechk = writechk;
   return 1;
}

extern void CbarDeInit(void)
{
   int i;

   CbarActivate(-1, NULL, NULL, 0);
   for (i=0; i<barcodes.n; i++)
      _CbarDestroyUnit(barcodes.units[i]);
   if (barcodes.units)
      Release(barcodes.units);
   barcodes.units = NULL;
   barcodes.n = 0;
}

/* The dialogue data
   -----------------*/

typedef struct t_dialbarc {
   struct t_dialunit **drs;   /* An array of reader dialogues (wrappers to
                                   reader-objects) */
   int org_sound_init;        /* Save origin state to ba able to restore it */
   int *confirmed;
   int n;                     /* Current number of readers in the dialogue */
   t_rsetting setting;        /* The items to be edited */
   int sel;                   /* Handle for the tab selection */
   int idtab;                 /* Handle to the tab-window */
   int idwin;                 /* Handle to the dialogue window */
   int dsound;                /* The flag "do include sound in the dialogue"
                                 from the appliation */
   int dsource;               /* The flag "do include source specification in
                                 the dialogue" from the appliation */
   const char *const*txt;
} t_dialbarc;

static void AddDialUnit(struct t_unit *bcu, t_dialbarc *bcd, int i)
{
   bcd->drs = ResizeMem(struct t_dialunit*, bcd->drs, i + 1);
   bcd->drs[i] = _CbarCreateDialUnit(bcu, bcd->dsource, bcd->dsound, &barcodes.setting);
}

static t_dialbarc *CreateDialBarcodeData(int dsound, int dsource, const char *const*txt)
{
   t_dialbarc *bcd;
   int i;

   bcd = GetMem0(t_dialbarc, 1);
   bcd->dsound = dsound;
   bcd->dsource = dsource;
   bcd->setting = barcodes.setting;
   bcd->n = barcodes.n;
   bcd->txt = txt;
   if (bcd->n < 1) {
      bcd->n = 1;
      AddDialUnit(NULL, bcd, 0);
   } else {
      for (i = 0; i < bcd->n; i++)
         AddDialUnit(barcodes.units[i], bcd, i);
   }
   return bcd;
}

static void DestroyDialogueData(t_dialbarc *bcd)
{
   int i;

   if (!bcd->org_sound_init)
      _CbarSetExternalSpeakers(0);
   for (i = barcodes.n - 1; i >= 0 ; i--)
      _CbarDestroyDialUnit(bcd->drs[i]);
   Release(bcd->drs);
   Release(bcd);
}

/* The dialogue
   ------------*/

static void UnitTab(void *data, int mb nouse)
{
   struct t_dialunit *dr = data;

   _CbarMakeUnitWidgets(dr);
}

static void ExtendDialUnits(void *data)
{
   t_dialbarc *bcd=data;
   char s[1000];

   AddDialUnit(NULL, bcd, bcd->n++);
   sprintf(s, "%s %d", bcd->txt[READER], bcd->n);
   AddTab(bcd->idtab, UnitTab, bcd->drs[bcd->n - 1], s);
   Refresh(bcd->idwin);
}

static int CheckSource(t_dialbarc *bcd)
{
   int i, j;

   for (i = 0; i < barcodes.n; i++)
      for (j = i + 1; j < barcodes.n; j++)
         if (!_CbarCheckUniqueSources(bcd->drs[i], bcd->drs[j]))
            return 0;
   return 1;
}

static void CancelSettingsDialogue(void *data)
{
   t_dialbarc *bcd = data;

   DestroyDialogueData(bcd);
   CloseWin(NULL);
   StopProcessEvents();
}

static void ExitSettinsDialogue(void *data)
{
   t_dialbarc *bcd = data;
   int i, logwin_open;

   if (!CheckSource(bcd))
      return;

   /*  Check new settings for each unit. */
   for (i = 0; i < bcd->n; i++)
      if (!_CbarCheckEditedUnit(bcd->drs[i]))
         return;

   /* Indicate for dialogue that user may have made changes */
   *bcd->confirmed = 1;

   /* Handle the error log control */
   if (bcd->setting.logging)
      CbarLogErrors(CBAR_LOG_ON);
   else
      CbarLogErrors(CBAR_LOG_OFF);
   logwin_open = barcodes.setting.logwin_open;

   /* Copy back edited data for all units */
   for (i = 0; i < bcd->n; i++)
      _CbarSetEditedData(bcd->drs[i]);
   barcodes.setting = bcd->setting;
   CancelSettingsDialogue(bcd);
   StoreSettings();
   if (logwin_open)
      CbarShowErrorLog();
}

/* Application interface (dialogue): */

extern int CbarSettingsDialogue(int len, int check, int dsound, int dsource,
       int errorlog, int *nrreaders)
{
   int i, nr, confirmed = 0;
   t_dialbarc *bcd;
   char s[1000];
   const char *const*txt;

   txt = LoadCompiledTexts(cbar_labels, "bcs", &nr);
   if (nr < SIZE_bcs)
      exit(-1);
   bcd = CreateDialBarcodeData(dsound, dsource, txt);
   bcd->confirmed = &confirmed;
   bcd->idwin = MkDialogue(ADAPTIVE, txt[OPTION_BAR_CODE_READER], 0);
   CguiLoadImage("cgui.dat", "barcode/browse", 0, bcd->idwin);

   if (len)
      AddEditBox(TOPLEFT, 30, txt[NO_DIGITS], FINT, 3, &bcd->setting.len);
   if (check) {
      AddCheck(DOWNLEFT, txt[PRINTING_CHECKSUM], &bcd->setting.writechk);
      AddCheck(RIGHT, txt[READING_CHECKSUM], &bcd->setting.readchk);
   }
   if (errorlog) {
      AddCheck(DOWNLEFT, txt[LOG_ERRORS], &bcd->setting.logging);
      AddCheck(RIGHT, txt[SHOW_LOG_WINDOW], &bcd->setting.logwin_open);
   }

   bcd->idtab = CreateTabWindow(DOWNLEFT | FILLSPACE | VERTICAL | HORIZONTAL,
                       ADAPTIVE, &bcd->sel);
   for (i = 0; i < bcd->n; i++) {
      sprintf(s, "%s %d", txt[READER], i + 1);
      AddTab(bcd->idtab, UnitTab, bcd->drs[i], s);
   }

   AddButton(DOWNLEFT, txt[ADD_UNIT], ExtendDialUnits, bcd);
   AddButton(DOWNLEFT, txt[OK_LABEL], ExitSettinsDialogue, bcd);
   AddButton(RIGHT, txt[CANCEL_LABEL], CancelSettingsDialogue, bcd);
   DisplayWin();
   _CbarSetExternalSpeakers(1);
   ProcessEvents();
   if (nrreaders)
      *nrreaders = barcodes.n;
   return confirmed;
}
